import './App.css';
import thunk from 'redux-thunk';
import { useEffect, useState } from 'react';
import { createStore, combineReducers, applyMiddleware } from 'redux';
import { Provider, connect } from 'react-redux';
import { Link, Route, Router, Switch, Redirect } from 'react-router-dom';
import createHistory from 'history/createBrowserHistory'

const actionPending = name => ({ type: 'PROMISE', status: 'PENDING', name })
const actionResolved = (name, payload) => ({ type: 'PROMISE', status: 'RESOLVED', name, payload })
const actionRejected = (name, error) => ({ type: 'PROMISE', status: 'REJECTED', name, error })

const actionPromise = (name, promise) =>
  async dispatch => {
    dispatch(actionPending(name))
    try {
      let payload = await promise
      dispatch(actionResolved(name, payload))
      return payload
    }
    catch (error) {
      dispatch(actionRejected(name, error))
    }
  }

const getGQL = url =>
  (query, variables = {}) =>
    fetch(url, {
      method: 'POST',
      headers: {
        "Content-Type": "application/json",
        ...(localStorage.authToken ? { "Authorization": "Bearer " + localStorage.authToken } :
          {})
      },
      body: JSON.stringify({ query, variables })
    })
      .then(res => res.json())
      .then(data => {
        if (data.errors && !data.data)
          throw new Error(JSON.stringify(data.errors))
        return data.data[Object.keys(data.data)[0]]
      })

const backendURL = "http://player.asmer.fs.a-level.com.ua"
const gql = getGQL(backendURL + '/graphql')
const history = createHistory()


function jwtDecode(token) {
  try {
    let decoded = token.split('.')
    decoded = decoded[1]
    decoded = atob(decoded)
    decoded = JSON.parse(decoded)
    return decoded

  } catch (e) {
    return;
  }
}

function authReducer(state, { type, token }) {
  if (!state) {
    if (localStorage.authToken) {
      type = 'AUTH_LOGIN'
      token = localStorage.authToken
    } else {
      return {}
    }
  }
  if (type === 'AUTH_LOGIN') {
    let auth = jwtDecode(token)
    if (auth) {
      localStorage.authToken = token
      return { token, payload: auth }
    }
  }
  if (type === 'AUTH_LOGOUT') {
    localStorage.authToken = ''
    return {}
  }

  return state
}

function promiseReducer(state = {}, { type, name, status, payload, error }) {
  if (type === 'PROMISE') {

    return {
      ...state,
      [name]: { status, payload: (state === 'PENDING' && state[name] && state[name].payload) || payload, error }
    }
  }
  return state
}

const localStoreReducer = (reducer, localStorageName) => {
  return (
    (state, action) => {
      let newState
      if(!state && localStorage.getItem(localStorageName)) {
        newState =  JSON.parse(localStorage.getItem(localStorageName))
        console.log(newState)
      } else {
        newState = reducer(state, action)
        localStorage.setItem(localStorageName, JSON.stringify(newState))
      }
      return newState 
    }
  )
}

const actionAuthLogin = (token) => ({ type: 'AUTH_LOGIN', token })
const actionAuthLogout = () => ({ type: 'AUTH_LOGOUT' })

const actionLogin = (login, password) =>
  actionPromise('login', gql(`
  query log($login:String!, $password:String!) {
      login(login: $login, password: $password)
    }`, { login, password }))

const actionFullLogin = (login = 'tst', password = '123') =>
  async dispatch => {
    let token = await dispatch(actionLogin(login, password))
    if (token) {
      await dispatch(actionAuthLogin(token))
      dispatch(actionGetUserData())
    }
  }

const actionRegister = (login, password) =>
  actionPromise('registration', gql(`
      mutation register($login:String!, $password:String!) {
          createUser(login: $login, password: $password) {
              login, _id
          }
      }
    `, { login, password }))

const actionFullRegister = (login = 'tst', password = '123') =>
  async dispatch => {
    await dispatch(actionRegister(login, password))
    await dispatch(actionFullLogin(login, password))
  }

const actionGetPlaylists = (userId = "5fe35e1ce926687ee86b0a3f") => {
  actionPromise('getPlaylists', gql(`
    query getPlaylistByOwnerId($ownerId:String!) {
      PlaylistFind(query: $ownerId) {
        _id, name
      }
    }
  `, { ownerId: JSON.stringify([{ ___owner: userId }]) }))
}

const actionGetUserData = () => {
  let _id = jwtDecode(localStorage.authToken).sub.id
  return (
    actionPromise('userData', gql(`
    query($userId: String!) {
      UserFindOne(query: $userId){
        login, _id, avatar {_id, url, originalFileName}
      }
    }
    `, { userId: JSON.stringify([{_id}]) }))

  )  
}

const store = createStore(
  combineReducers(
    {
      promise: promiseReducer,
      auth: authReducer,
      //local: localStoreReducer(promiseReducer, 'locale')
    }
  ), applyMiddleware(thunk)
)
store.subscribe(() => console.log(store.getState()))
if(store.getState().auth?.token) {
  //history.push('/player')
  store.dispatch(actionGetUserData())
} else {
  history.push('/login')
}


const LoginForm = ({ loged, onLogin }) => {
  let [login, setLogin] = useState()
  let [password, setPassword] = useState()
  let [log, setLog] = useState()

  useEffect(() => {
    setLog(loged)
    if (log?.payload && localStorage.authToken) history.push('/player')
  }, [loged, log])

  return (
    <>
      <h1>Web-player</h1>
      <div>
        <h2>Log-in</h2>
        <input type="text" placeholder='Login' onChange={(e) => setLogin(e.target.value)} />
        <br />
        <input type="password" placeholder='Password' onChange={(e) => setPassword(e.target.value)} />
        <br />
        <small style={{ color: 'red' }}>{loged.status === 'REJECTED' || (loged.status === 'RESOLVED' && !loged.payload) ? 'invalid login or password' : ''}</small>
        <br />
        <button
          disabled={!password || !login}
          onClick={() => { onLogin(login, password) }}
        >Login</button>
        <p>- OR -</p>
        <Link to="/registration">Register new user</Link>
      </div>
    </>
  )
}

const LoginFormConnect = connect(state => ({ loged: state.promise.login || {} }), { onLogin: actionFullLogin })(LoginForm)

const RegisterForm = ({ onRegister }) => {
  let [login, setLogin] = useState()
  let [password, setPassword] = useState()
  let [password2, setPassword2] = useState()

  return (
    <>
      <h1>Web-player</h1>
      <div>
        <h2>Registration</h2>
        <input type="text" placeholder='Login' onChange={(e) => setLogin(e.target.value)} />
        <br />
        <input type="password" placeholder='Password' onChange={(e) => setPassword(e.target.value)} />
        <br />
        <input disabled={!password} type="password" placeholder='Repeat Password' onChange={(e) => setPassword2(e.target.value)} />
        <br />
        <small style={{ color: 'red' }}>{password2 && password2 !== password ? 'Passwords do not match' : ''}</small>
        <br />
        <button disabled={!password || !login || password2 !== password} onClick={() => onRegister(login, password)}>Register</button>
        <br />
        <Link to="/login">Back to log-in page</Link>
      </div>
    </>
  )
}
const RegisterFormConnect = connect(null, { onRegister: actionFullRegister })(RegisterForm)

const Player = ({ user, getUserData, onLogout }) => {
  let [userInfo, setUserInfo] = useState(user.payload)
  useEffect(()=> {
    //setUserInfo(userData.payload)
    setUserInfo(user.payload)
    //console.log('data & info', user.payload, userInfo)
  },[user, userInfo])
  
  return (
    <>
      <header>Player</header>
      <div style={{ display: 'flex' }}>
        <aside style={{ border: '1px solid black', width: '20%' }}>
          <div
            style={{ border: '1px solid black', backgroundColor: 'red', color: 'white' }}
            onClick={() => { onLogout(); history.push('/login') }}
          >
            log-out[X]
          </div>

          <div style={{ display: 'flex', alignItems: 'center' }}>
            {/* avatar */}
            <img 
              width={100} 
              height={100}  
              style={{ border: '1px solid black' }}
              src={ userInfo?.avatar?.url ? backendURL + '/' + userInfo?.avatar?.url : ''}
              alt='avatar' 
            />
    
            <div>
              <span>{userInfo?.login || 'user'}</span>
            </div>
          </div>
          <div style={{ border: '1px solid black' }}>
            <h4>User playlists:</h4>
            <div style={{ border: '1px solid black' }}>Add plylist [+]</div>
            <ul>
              тут карочи линки будут
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
            </ul>
          </div>
        </aside>
        <main style={{ border: '1px solid black', width: '80%' }}>
          MAIN
          рут патх='путь' компонет=`компогнент отривоски`
        </main>
      </div>
      <footer> back stop forw</footer>
    </>
  )
}

const PlayerConnect = connect(
  state => ({
     //playlists: state.promise.getPlaylists || {},
     user: state.promise.userData || {}
  }),
  { 
    onLogout: actionAuthLogout,
    getUserData: actionGetUserData,
    //getPlaylists: actionGetPlaylists
  }
)(Player)


function App() {
  //if(store.getState().auth?.token) history.push('/player')
  return (
    <Router history={history}>
      <Provider store={store}>
        <div className="App">
          <Switch>
            <Route path="/login" component={LoginFormConnect} exact />
            <Route path="/registration" component={RegisterFormConnect} exact />
            <Route path="/player" component={PlayerConnect} exact />
          </Switch>
        </div>
      </Provider>
    </Router>
  );
}

export default App;

